// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: option.h 107 2008-02-04 14:46:27Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef OPTION_H
#define OPTION_H

#include <vector>
#include <string>
#include <ostream>

enum heuristic_opt {
    NULL_HEURISTIC = 0,
    GD_SUM_HEURISTIC,
    GD_SUM_USELESS_HEURISTIC,
    GD_MAX_HEURISTIC,
    GD_MAX_USELESS_HEURISTIC,
    H_MAX_HEURISTIC,
    H_MAX_USELESS_HEURISTIC,
    RANDOM_HEUR_HEURISTIC,
    PDB_HEURISTIC
};

enum cache_opt {
    NO_CACHE = 0,
    BITSTATE_CACHE,
    DISCRETE_CACHE,
    WHOLE_STATE_CACHE
};

enum search_opt {
    BFS_SEARCH = 0,
    DFS_SEARCH,
    GREEDY_SEARCH,
    A_STAR_SEARCH
};

enum trace_opt {
    NO_TRACE = 0, 
    STATISTIC_TRACE,
    SHOW_TRACE,	  
    SYMBOLIC_TRACE
};

////////////////////////////////////////////////////////////////////

class Options {
private:
    void parseCmdLine(int argc, char** argv);
    
public:
    heuristic_opt heuristic;
    cache_opt cache;
    search_opt search;
    trace_opt trace;
    bool quiet;
    bool memtime;
    bool batch_mode;
    bool new_syntax;
    std::string ta_file;
    std::string q_file;
    
    Options(int argc, char** argv) {
	// set default values
	heuristic    = NULL_HEURISTIC;
	cache        = NO_CACHE;
	search       = BFS_SEARCH;
	trace        = STATISTIC_TRACE;
	quiet        = false;
	memtime      = true;
	batch_mode   = false;
	new_syntax   = false;

	parseCmdLine(argc, argv);
    }
};

std::ostream& operator<<(std::ostream& o, const Options& opts);

#endif /* OPTION_H */
