// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: ffassign.h 68 2008-01-14 09:09:37Z sebastian $
//
//////////////////////////////////////////////////////////////////////

#ifndef FF_ASSIGN_H
#define FF_ASSIGN_H

#include "ff_heuristic/ffvar.h"
#include <inttypes.h>
#include <iosfwd>

namespace ff {
    
    //////////////////////////////////////////////////////////////////////
    //
    // An Assignment is the base class of all assignments. So far
    // there are only two types of assignemnts x := y and x:= c where
    // x, y are integer variables and c ins a constant. The apply
    // function returns true if the assignment can be applied multiple
    // times
    //
    //////////////////////////////////////////////////////////////////////

    class IntegerAssignment {
    protected:
	int32_t id;
	IntegerAssignment(int32_t id) : id(id) {}       
    public:
	virtual ~IntegerAssignment() {}
	virtual int32_t getID() const { return id; }
	virtual bool apply() = 0;
	virtual bool idempotent() const = 0;
	virtual std::ostream& display(std::ostream& o) const = 0;
    };

    inline std::ostream& operator<<(std::ostream& o, const IntegerAssignment& ass) {
	return ass.display(o);
    }

    ////////////////////////////////////////////////////////////////////
    
    class VarAssignment : public IntegerAssignment {
    private:
	Variable* lhs;
	Variable* rhs;
    public:
	VarAssignment(int32_t id, Variable* lhs, Variable* rhs);
	virtual bool apply();
	virtual bool idempotent() const { return false; }
	virtual std::ostream& display(std::ostream& o) const;
    };

    inline bool VarAssignment::apply() {
	return lhs->add(rhs);
    }

    ////////////////////////////////////////////////////////////////////
    
    class ConstAssignment : public IntegerAssignment {
    private:
	Variable* lhs;
	int32_t rhs;
    public:
	ConstAssignment(int32_t id, Variable* lhs, int32_t rhs);
	virtual bool apply();
	virtual bool idempotent() const { return true; }
	virtual std::ostream& display(std::ostream& o) const;
    };

    inline bool ConstAssignment::apply() {
	return lhs->add(rhs);
    }

    ////////////////////////////////////////////////////////////////////

    class NullAssignment : public IntegerAssignment {
    public:
	NullAssignment() : IntegerAssignment(-1) {}
	virtual bool apply() { return false; }
	virtual bool idempotent() const { return true; }
	virtual std::ostream& display(std::ostream& o) const;
    };
}

#endif /* FF_ASSIGN_H */
