// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: ffiguard.cpp 68 2008-01-14 09:09:37Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "ff_heuristic/ffiguard.h"
#include "ff_heuristic/ffvar.h"
#include "ff_heuristic/ffhelper.h"

#include <inttypes.h>
#include <iostream>
#include <cassert>

namespace ff {
    
    using namespace std;

    VarConstraint::VarConstraint(const Variable* lhs, const Variable* rhs, comp_t comp) : 
	lhs(lhs),
	rhs(rhs),
	comp(comp)
    {}
    
    ostream& VarConstraint::display(ostream& o) const {
	return o << lhs->getName() << " = " << rhs->getName();
    }
    
    ////////////////////////////////////////////////////////////////////

    ConstConstraint::ConstConstraint(const Variable* lhs, int32_t rhs, comp_t comp) : 
	lhs(lhs), 
	rhs(rhs),
	comp(comp)
    {}
	
    ostream& ConstConstraint::display(ostream& o) const {
	return o << lhs->getName() << " = " << rhs;
    }

    ////////////////////////////////////////////////////////////////////
    
    NullConstraint::NullConstraint() {}

    ostream& NullConstraint::display(std::ostream& o) const { 
	return o << "true";
    }
    
    ////////////////////////////////////////////////////////////////////

    IntegerGuard::IntegerGuard() : 
	constraints(),
	firstUnsat(0) 
    {}

    IntegerGuard::~IntegerGuard() {
	destroyVector(constraints);
    }

    void IntegerGuard::addConstraint(const IntegerConstraint* ic) {
	assert(ic != 0);
	constraints.push_back(ic);
    }
	
    ostream& IntegerGuard::display(ostream& o) const {
	if (empty()) {
	    return o << "true";
	} else {
	    for (uint32_t i = 0; i < constraints.size() - 1; i++) {
		o << *constraints[i] << " && ";
	    }
	    return o << *constraints.back();
	}
    }
}

