// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: ffvar.h 68 2008-01-14 09:09:37Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef FF_VAR_H // don't change the name of the define since it is
		 // also used in ffvar.cpp
#define FF_VAR_H

#include <inttypes.h>
#include <iosfwd>
#include <string>

namespace ff {
    ////////////////////////////////////////////////////////////////////   
    //    
    // A Value has a current and a next value. New values are added to
    // the next value and become available after the next call to
    // incLevel().
    // 
    ////////////////////////////////////////////////////////////////////   
    
    class Value {
    private:
	static const int32_t MIN = 0;
	static const int32_t MAX = 31;

	typedef uint32_t raw_t;
	raw_t current;
	raw_t next;

	raw_t lastBit(raw_t bits) const; 
	raw_t nextBit(raw_t& bits)const; 
	raw_t int2raw(int32_t value) const; 	
	int32_t raw2int(raw_t bits) const; 
	int32_t nextInt(raw_t& bits) const; 
	
    public:
	Value();
	
	void init(int32_t val); 
	void incLevel(); 
	bool add(const Value& val);
	bool add(int val); 

	bool lt(const Value& val) const; 
	bool lt(int val) const;	
	bool le(const Value& val) const; 
	bool le(int val) const;	
	bool eq(const Value& val) const; 
	bool eq(int val) const;	
	bool ge(const Value& val) const; 
	bool ge(int val) const;	
	bool gt(const Value& val) const; 
	bool gt(int val) const;	
	bool neq(const Value& val) const; 
	bool neq(int val) const;
	
	std::ostream& display(std::ostream& o) const;
    };

    inline std::ostream& operator<<(std::ostream& o, const Value& val) {
	return val.display(o);
    }

    ////////////////////////////////////////////////////////////////////
    //
    // A Range has a lower and an upper bound. 
    //
    ////////////////////////////////////////////////////////////////////
    
    class Range {
    public:
	int32_t lower;
	int32_t upper;
	
	Range(int32_t lower, int32_t upper) : lower(lower), upper(upper) {}
	std::ostream& display(std::ostream& o) const;
    };
    
    inline std::ostream& operator<<(std::ostream& o, const Range& range) {
	return range.display(o);
    }

    ////////////////////////////////////////////////////////////////////
    //
    // A Variable consists of a Range and a Value object. 
    // 
    ////////////////////////////////////////////////////////////////////

    class Variable {
    private:
	std::string name;
	Range range;
	Value val;
    public:
	Variable();

	void init(int32_t value); 
	void incLevel(); 
	bool add(const Variable* other); 
	bool add(int32_t value); 
	
	// comparators
	bool lt(const Variable* other) const; 
	bool lt(int32_t value) const;

	bool le(const Variable* other) const; 
	bool le(int32_t value) const;

	bool eq(const Variable* other) const; 
	bool eq(int32_t value) const;

	bool ge(const Variable* other) const; 
	bool ge(int32_t value) const;

	bool gt(const Variable* other) const; 
	bool gt(int32_t value) const;

	bool neq(const Variable* other) const; 
	bool neq(int32_t value) const;

	void setRange(const Range& r);
	const Range& getRange() const; 
	void setName(const std::string& name);
	const std::string& getName() const;
	std::ostream& display(std::ostream& o) const;
    };

    inline std::ostream& operator<<(std::ostream& o, const Variable& var) {
	return var.display(o);
    }
}

#include "ff_heuristic/ffvar.cpp"

#endif /* FF_VAR_H */
