// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: gd.h 83 2008-01-16 12:44:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef GD_HEURISTIC_H
#define GD_HEURISTIC_H

#include "search/heuristic.h"
#include "gd_heuristic/matrix.h"
#include <inttypes.h>
#include <vector>

class Task;
class State;
class Matrix;

class GDHeuristic : public Heuristic {
protected:
    const Task* task;
    std::vector<Matrix> matrices;

    virtual void update(int32_t& result, int32_t value) const = 0;
public:
    GDHeuristic(const Task* task);
    virtual ~GDHeuristic() {}

    virtual void init(const std::vector<bool>& abstracted);
    virtual int32_t eval(const State* state);
};

////////////////////////////////////////////////////////////////////

class GDMAXHeuristic : public GDHeuristic {
private:
    void update(int32_t& result, int32_t value) const { result = std::max(result, value); }    
public:
    GDMAXHeuristic(const Task* task);
};

////////////////////////////////////////////////////////////////////

class GDSUMHeuristic : public GDHeuristic {
private:    
    void update(int32_t& result, int32_t value) const { result += value; }    
public:
    GDSUMHeuristic(const Task* task);
};

#endif /* MDH_MD_HEURISTIC_H */
