// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: abststate.cpp 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "pdb_heuristic/abststate.h"
#include "pdb_heuristic/abstsys.h"
#include "system/zone.h"
#include "system/task.h"
#include <cassert>
#include <iostream>
#include <iomanip>

using namespace std;

namespace pdb {
    
    AbstractSystem* AbstractState::system = NULL;

    AbstractState::AbstractState() :
	locations(), 
	intvalues(), 
	distance(-1)
    {
	assert(system);	
    }

    AbstractState::AbstractState(AbstractSystem* sys) : 
	locations(), 
	intvalues(), 
	distance(-1)
    {
	assert(sys);
	system = sys;
    }
    
    void AbstractState::addClockConstraint(int32_t i, int32_t j, raw_t bound) {
	if (zone.isEmpty()) {
	    zone.setDimension(system->getNrClocks());
	    zone.setInit();	  
	}
	if (i != j) {
	    zone.constrain(i, j, bound >> 1, bound & 1);	
	}
    }

    int32_t AbstractState::hashKey() const {
	// rotating hash
	int32_t result = 0;
	for (uint32_t i = 0; i < locations.size(); i++) {
	    result = ( result << 4 ) ^ ( result >> 28 ) ^ locations[i];
	}
	
	for (uint32_t i = 0; i < intvalues.size(); i++) {
	    result = ( result << 4 ) ^ ( result >> 28 ) ^ intvalues[i];
	}	
	return result;
    }
    
    ostream& AbstractState::display(ostream& o) const {
	assert(system != 0);
	o << "( ";
	for (uint32_t i = 0; i < locations.size(); i++) {
	    o << system->getProcess(i)->getLocation(locations[i])->getName() << " ";
	}
	o << ")" << endl;
	dump(o, zone);
	//o << zone;
	o << ": ";
	for (uint32_t i = 0; i < intvalues.size(); i++) {
	    o << system->getInteger(i)->getName() << "=" << intvalues[i] << " ";
	}

	return o << ": " << distance;
    }

    bool AbstractState::discEqual(const AbstractState& other) const {
	return locations == other.locations && intvalues == other.intvalues;
    }
}
    

