// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: abstsys.cpp 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "pdb_heuristic/abstsys.h"
#include "system/task.h"
#include <cassert>

using namespace std;

namespace pdb {
    
    SystemComponent::SystemComponent(const string name, int32_t id) : 
	name(name), 
	id(id) 
    {}
    
    SystemComponent::~SystemComponent() 
    {}
    
    string SystemComponent::getName() const { 
	return name; 
    }
    
    void SystemComponent::setName(const string name) { 
	this->name = name; 
    }
    
    int32_t SystemComponent::getId() const { 
	return id; 
    }
    
    void SystemComponent::setId(int32_t id) { 
	this->id = id; 
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////

    Location::Location(const string& name, Process* proc, int32_t id) : 
	SystemComponent(name, id), proc(proc) {}

    string Location::getName() const {
	return proc->getName() + "." + name;
    }
    
    ostream& Location::display(ostream& o) const {
	return o << "loc " << id << ": " << proc->getName() << "." << name;
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    
    Process::Process() : 
	SystemComponent("", 0),
	locations()
    {}
    
    void Process::addLocation(Location* loc) {
	locations.push_back(loc);
    }

    const Location* Process::getLocation(uint32_t id) const {
	assert(id < locations.size());
	return locations[id];
    }

    ostream& Process::display(ostream& o) const {
	o << "proc " << id << ": " << name << endl;
	for (uint32_t i = 0; i < locations.size(); i++) {
	    o << "    " << *locations[i] << endl;
	}
	return o;
    }

    ///////////////////////////////////////////////////////////////////////////////////////

    Integer::Integer(const string& name) :
	SystemComponent(name, -1) {}
    
    ostream& Integer::display(ostream& o) const {
	return o << "int " << id << ": " << name;
    }

    ///////////////////////////////////////////////////////////////////////////////////////

    Clock::Clock(const string& name) :
	SystemComponent(name, -1) {}
    
    ostream& Clock::display(ostream& o) const {
	return o << "clock " << id << ": " << name;
    }	

    ///////////////////////////////////////////////////////////////////////////////////////

    AbstractSystem::AbstractSystem() : 
	processes(), 
	locations(), 
	integers() 
    {}
    
    void AbstractSystem::addProcess(Process* proc) {
	proc->setId(processes.size());
	processes.push_back(proc);
    }
    
    void AbstractSystem::addInteger(Integer* integer) {
	integer->setId(integers.size());
	integers.push_back(integer);
    }
    
    void AbstractSystem::addClock(Clock* clock) {
	clock->setId(clocks.size());
	clocks.push_back(clock);
    }

    const Process* AbstractSystem::getProcess(uint32_t id) const {
	assert(id < processes.size());
	return processes[id];
    }

    const Location* AbstractSystem::getLocation(uint32_t id) const {
	assert(id < locations.size());
	return locations[id];
    }
    
    const Integer* AbstractSystem::getInteger(uint32_t id) const {
	assert(id < integers.size());
	return integers[id];
    }
    
    const Clock* AbstractSystem::getClock(uint32_t id) const {
	assert(id < clocks.size());
	return clocks[id];
    }
    
    ostream& AbstractSystem::display(ostream& o) const {
	for (uint32_t i = 0; i < integers.size(); i++) {
	    o << *integers[i] << endl;
	}
	for (uint32_t i = 0; i < clocks.size(); i++) {
	    o << *clocks[i] << endl;
	}
	o << endl;
	for (uint32_t i = 0; i < processes.size(); i++) {
	    o << *processes[i] << endl;
	}
	return o;
    }    
}
