// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: factory.h 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef PDB_FACTORY_H
#define PDB_FACTORY_H

#include <ext/hash_map>
#include <string>
#include <iostream>
#include <inttypes.h>

namespace __gnu_cxx {
    template<> struct hash<std::string> {
	inline size_t operator()(const std::string& str) const {
	    size_t res = 0;
	    for (uint32_t i = 0; i < str.size(); i++) {
		res *= 33;
		res ^= static_cast<size_t>(str[i]);
	    }
	    return res;
	}
    };
}

namespace pdb {
    
    template<class T>
    class Factory {
    private:    
	typedef typename __gnu_cxx::hash_map<std::string, int32_t> hash_t;
	typedef typename std::vector<T*> store_t;
	hash_t hash;
	store_t store;
	
    public:
	typedef typename hash_t::const_iterator const_iterator;
	typedef typename hash_t::iterator iterator;    
	
	Factory() : hash(), store() {}       
	
	~Factory() {
	    for (uint32_t i = 0; i < store.size(); i++) {
		delete store[i];
	    }
	}

	inline const T* get(int32_t id) const {
	    return store[id];
	}
	
	inline T* get(const std::string& name) {
	    std::pair<iterator, bool> pos = hash.insert(make_pair(name, 0));
	    if (pos.second) {
		int32_t index = store.size();
		store.push_back(new T(name, index));
		pos.first->second = index;
	    }
	    return store[pos.first->second];
	}

	inline const T* find(const std::string& name) const {
	    const_iterator it = hash.find(name);
	    if (it != hash.end()) {
		return store[it->second];
	    } else {
		return static_cast<const T*>(0);
	    }
	}

	inline size_t size() const {
	    return store.size();
	}
	
	inline std::ostream& display(std::ostream& o) const {
	    for (uint32_t i = 0; i < store.size(); i++) {
		o << *store[i] << " ";
	    }
	    return o;
	}
	
	inline iterator begin() { return hash.begin(); }
	inline iterator end() { return hash.end(); }
	
	inline const_iterator begin() const { return hash.begin(); }
	inline const_iterator end() const { return hash.end(); }
    };
    
    template<class T>
    inline std::ostream& operator<<(std::ostream& o, const Factory<T>& factory) {
	return factory.display(o);
    }   
}
#endif /* PDB_FACTORY_H */
    
