// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: hash.cpp 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "pdb_heuristic/hash.h"
#include "pdb_heuristic/abststate.h"
#include <climits>

namespace pdb {
    
    using namespace std;
   
    IntersectionHash::IntersectionHash() : 
	Hash(),
	hash()
    {}

    int32_t IntersectionHash::lookup(const AbstractState* state) const {
	int32_t result = INT_MAX;
	hash_t::const_iterator pos = hash.find(state->hashKey());
	if (pos == hash.end()) {
	    return result;
	}
	const list_t& l = pos->second;
	for (list_t::const_iterator it = l.begin(); it != l.end(); ++it) {
	    if ((*it)->discEqual(*state) && (*it)->zone.intersects(state->zone)) {
		assert((*it)->distance != -1);
		result = std::min(result, (*it)->distance);
	    }
	}
	return result;
    }
    
    void IntersectionHash::insert(AbstractState* state) {
	pair<hash_t::iterator, bool> pos = hash.insert(make_pair(state->hashKey(), list_t()));
	list_t& l = pos.first->second;
	l.push_front(state);
    }

    void IntersectionHash::clear() {
	for (hash_t::iterator h = hash.begin(); h != hash.end(); ++h) {
	    list_t& l = h->second;
	    for (list_t::iterator it = l.begin(); it != l.end(); ++it) {
		delete *it;
	    }
	    l.clear();
	}
	hash.clear();
    }    

    void IntersectionHash::dump() const {
	for (hash_t::const_iterator h = hash.begin(); h != hash.end(); ++h) {
	    const list_t& l = h->second;
	    for (list_t::const_iterator it = l.begin(); it != l.end(); ++it) {
		cout << (*it)->hashKey() << endl;
		(*it)->display(cout);
		cout << endl << endl;
	    }
	}
    }
}
