// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: hash.h 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef PDB_HASH_H
#define PDB_HASH_H

#include "pdb_heuristic/abststate.h"
#include <ext/hash_map>
#include <ext/slist>

namespace pdb {
    class AbstractState;
}

namespace __gnu_cxx {
    template<> 
    struct hash<pdb::AbstractState*> {
	inline int32_t operator()(const pdb::AbstractState* state) const {
	    return state->hashKey();
	}
    };
}

namespace pdb {
    class Hash {
    protected:
	Hash() {}
    public:
	virtual ~Hash() { clear(); }
	virtual int32_t lookup(const AbstractState* state) const = 0;
	virtual void insert(AbstractState* state) = 0;
	virtual void clear() {}
	virtual void dump() const {};
    };   

    ////////////////////////////////////////////////////////////////////

    class IntersectionHash : public Hash {
    private:
	typedef __gnu_cxx::slist<AbstractState*> list_t; 
	typedef __gnu_cxx::hash_map<int32_t, list_t> hash_t;
	
	hash_t hash;
    public:
	IntersectionHash();
	
	virtual int32_t lookup(const AbstractState* state) const;
	virtual void insert(AbstractState* state);
	virtual void clear();
	virtual void dump() const;
    };    
}

#endif /* PDB_HASH_H */
