////////////////////////////////////////////////////////////////////
//
// $Id: parser.yy 101 2008-01-22 13:17:14Z sebastian $
//
////////////////////////////////////////////////////////////////////

%{
#include <cstdio>
#include <iostream>
#include <string>
#include "pdb_heuristic/systembuilder.h"
#include "pdb_heuristic/pdb-builder.h"
        
int yylex();
int yyparse();    
void yyerror(char* s);
pdb::AbstractSystemBuilder sys_builder;
pdb::PDBBuilder pdb_builder(sys_builder.getSystem()); 

%}

%union {
    int number;
    char* str;
}

%token<number> INT;
%token PROCESSES INTEGERS CLOCKS;
%token<str> ID;

%%
file          : prelude_list state_list
              ;

prelude_list  : prelude_list prelude 
              | prelude
              ;

prelude       : PROCESSES ID ':' loc_names ';' { sys_builder.addProcess($2); free($2); }
              | INTEGERS int_names ';'
              | CLOCKS clock_names ';'
              ;

loc_names     : loc_names ID { sys_builder.addLocation($2); free($2); }
              | 
              ;  

int_names     : int_names ID { sys_builder.addInteger($2); free($2); }
              | 
              ; 

clock_names   : clock_names ID { sys_builder.addClock($2); free($2); }
              | 
              ;

state_list    : state_list state
              | state
              ;

state         : location_list '.' clock_list '.' integer_list '.' ':' INT { pdb_builder.addDistance($8); }
              ;

location_list : location_list INT { pdb_builder.addLocation($2); }
              | 
              ;

clock_list    : clock_list INT INT INT '.' { pdb_builder.addClockConstraint($2, $3, $4); }
              | 
              ;

integer_list  : integer_list INT  { pdb_builder.addIntegerValue($2); } 
              | 
              ;
%%

#include "pdb_heuristic/lexer.cc"

void yyerror(char* s) {
    std::printf("%d %s\n", yylineno, s);
}

pdb::PDBBuilder& parse(const char* filename) {
    yyin = fopen(filename, "r");
    if (yyin == 0) {
	std::printf("Error: file '%s' not found! Aborting...\n", filename);
	exit(EXIT_FAILURE);
    } else {
	yyparse();
    }
    return pdb_builder;
}
