// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: pdb-builder.cpp 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "pdb_heuristic/pdb-builder.h"
#include "pdb_heuristic/abstsys.h"
#include "pdb_heuristic/abststate.h"
#include "pdb_heuristic/hash.h"

#include <iostream>
#include <cassert>

using namespace std;

namespace pdb {
    
    PDBBuilder::PDBBuilder(AbstractSystem* system) : 
	system(system),
	state(new AbstractState(system)),
	hash(new IntersectionHash())
    {
	assert(state);
    }
    
    void PDBBuilder::addDistance(int32_t dist) {
	assert(state);
	state->distance = dist;
	hash->insert(state);
	state = new AbstractState(system);	
    }

    int32_t PDBBuilder::lookup(const AbstractState* astate) const {
	return hash->lookup(astate);
    }

    void PDBBuilder::dump() const {
	hash->dump();
	exit(0);	
    }
    
    AbstractSystem* PDBBuilder::getSystem() const {
	return system;
    }

    void PDBBuilder::addLocation(int32_t loc) {	
	state->locations.push_back(loc);
    }
    
    void PDBBuilder::addIntegerValue(int32_t val) {
	state->intvalues.push_back(val);
    }

    void PDBBuilder::addClockConstraint(int32_t i, int32_t j, int32_t bound) {
	if (i != j) {
	    state->addClockConstraint(i, j, bound);
	}
    }
    
    ostream& PDBBuilder::display(ostream& o) const {
	return o;
    }
}
