// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: pdb.cpp 100 2008-01-22 13:15:55Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "pdb_heuristic/pdb.h"
#include "pdb_heuristic/pdb_parser.h"
#include "pdb_heuristic/pdb-builder.h"
#include "pdb_heuristic/abstsys.h"
#include "pdb_heuristic/abststate.h"

#include "system/system.h"
#include "system/task.h"
#include "system/process.h"
#include "system/variable.h"

#include <string>
#include <iostream>

namespace pdb {
    
    using namespace std;
    
    ////////////////////////////////////////////////////////////////////

    template<class A>
    static void init_procs(const ::System* system, const vector<A*>& abstract, vector<uint32_t>& map) {
	for (uint32_t i = 0; i < system->getNrProcesses(); i++) {
	    for (uint32_t j = 0; j < abstract.size(); j++) {
		if (system->procs[i]->name == abstract[j]->getName()) {
		    assert((uint32_t)abstract[j]->getId() < map.size());
		    map[abstract[j]->getId()] = system->procs[i]->id;
		    break;
		}
	    }
	}
    } 
    
    template<class A>
    static void init_clocks(const ::System* system, const vector<A*>& abstract, vector<uint32_t>& map) {
	for (uint32_t i = 0; i < system->getNrClocks(); i++) {
	    for (uint32_t j = 0; j < abstract.size(); j++) {
		if (system->clocks[i]->name == abstract[j]->getName()) {
		    assert((uint32_t)abstract[j]->getId() < map.size());
		    map[abstract[j]->getId()] = system->clocks[i]->id;
		    break;
		}
	    }
	}
	
	for (uint32_t i = 0; i < system->getNrProcesses(); i++) {
	    const ::Process* proc = system->getProcess(i);
	    for (uint32_t j = 0; j < proc->getNrClocks(); i++) {
		if (proc->clocks[j]->name == abstract[j]->getName()) {
		    assert((uint32_t)abstract[j]->getId() < map.size());
		    map[abstract[j]->getId()] = system->clocks[i]->id;
		    break;
		}
	    }
	}
    }

    template<class A>
    static void init_ints(const ::System* system, const vector<A*>& abstract, vector<uint32_t>& map) {
	for (uint32_t i = 0; i < system->getNrIntegers(); i++) {
	    for (uint32_t j = 0; j < abstract.size(); j++) {
		if (system->ints[i]->name == abstract[j]->getName()) {
		    assert((uint32_t)abstract[j]->getId() < map.size());
		    map[abstract[j]->getId()] = system->ints[i]->id;
		    break;
		}
	    }
	}
	
	for (uint32_t i = 0; i < system->getNrProcesses(); i++) {
	    const ::Process* proc = system->getProcess(i);
	    for (uint32_t j = 0; j < proc->getNrClocks(); i++) {
		if (proc->clocks[j]->name == abstract[j]->getName()) {
		    assert((uint32_t)abstract[j]->getId() < map.size());
		    map[abstract[j]->getId()] = system->ints[i]->id;
		    break;
		}
	    }
	}
    }
    
    ////////////////////////////////////////////////////////////////////
    
    PDBHeuristic::PDBHeuristic(const Task* task) :
	builder(parse("./c.lst"))
    {
	AbstractSystem* asys = builder.getSystem();
	asys->task = task;
	proc_map.resize(asys->getNrProcesses());
	var_map.resize(asys->getNrIntegers());
	clock_map.resize(asys->getNrClocks());
	
	init_procs(task->system, asys->getProcesses(), proc_map);
	init_clocks(task->system, asys->getClocks(), clock_map);
	init_ints(task->system, asys->getIntegers(), var_map);
	
#ifndef NDEBUG	
	for (uint32_t i = 0; i < proc_map.size(); i++) {
	    assert(asys->getProcess(i)->getName() == task->system->getProcess(proc_map[i])->name);
	}
	for (uint32_t i = 0; i < var_map.size(); i++) {
	    assert(asys->getInteger(i)->getName() == task->system->getInteger(var_map[i])->name);
	}
	for (uint32_t i = 0; i < clock_map.size(); i++) {
	    assert(asys->getClock(i)->getName() == task->system->getClock(clock_map[i])->name);
	}
#endif
	//builder.dump();
    }

    void PDBHeuristic::abstract(AbstractState& astate, const State* state) const {
	assert(astate.locations.empty() && astate.intvalues.empty());
	for (uint32_t i = 0; i < proc_map.size(); i++) {
	    astate.locations.push_back(state->getLocationNr(proc_map[i]));
	}
	for (uint32_t i = 0; i < var_map.size(); i++) {
	    astate.intvalues.push_back(state->getIntegerValue(var_map[i]));
	}
 	for (uint32_t i = 0; i < clock_map.size(); i++) {
	    for (uint32_t j = 0; j < clock_map.size(); j++) {
		// int32_t bound = state->zone()(clock_map[i], clock_map[j]) >>  1;
		// bound |= (state->zone()(clock_map[i], clock_map[j]) & 1) ? 1 : 0;
		//cout << clock_map[i] << " " <<  clock_map[j] << endl;
		//if ( clock_map[i] != 0 || clock_map[j] != 0) {
		    astate.addClockConstraint(i, j, state->zone()(clock_map[i], clock_map[j]));
		    //}
		//astate.addClockConstraint(i, j, bound);
	    }
 	}
	astate.zone = state->zone();
    }
    
    int32_t PDBHeuristic::eval(const State* state) {
	AbstractState astate;
	abstract(astate, state);
	//cout << astate << "PF" << endl;
	return builder.lookup(&astate);
    }
}
