// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id$
//
////////////////////////////////////////////////////////////////////

#ifndef CACHE_H
#define CACHE_He

#include "search/heuristic.h"
#include <vector>
#include <ext/slist>
#include <ext/hash_map>

class State;

class HeuristicCache : public Heuristic {
protected:
    Heuristic* heuristic;
    uint32_t size;
public:
    HeuristicCache(Heuristic* heuristic, uint32_t size); 
};

////////////////////////////////////////////////////////////////////

class BitstateCache : public HeuristicCache {
private:
    static const int32_t DONT_KNOW = -1;
    typedef std::vector<int32_t> cache_t;
    cache_t cachedStates;
public:
    BitstateCache(Heuristic* heuristic, uint32_t size);
    virtual int32_t eval(const State* state);
};

////////////////////////////////////////////////////////////////////

class DiscretePartCache : public HeuristicCache {
private:
    typedef std::pair<const State*, int32_t> entry_t;
    typedef __gnu_cxx::slist<entry_t> list_t;
    typedef __gnu_cxx::hash_map<uint32_t, list_t> cache_t;
    
    cache_t cachedStates;
protected:
    virtual bool isEqual(const State* lhs, const State* rhs) const;
public:
    DiscretePartCache(Heuristic* heuristic, uint32_t size);
    virtual int32_t eval(const State* state);
};

////////////////////////////////////////////////////////////////////

class WholeStateCache : public DiscretePartCache {
private:
    virtual bool isEqual(const State* lhs, const State* rhs) const;
public:
    WholeStateCache(Heuristic* heuristic, uint32_t size);
};

#endif /* CACHE_H */
