// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: closedlist.cpp 68 2008-01-14 09:09:37Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "search/closedlist.h"
#include "system/state.h"
#include <utility>

using namespace std;

ClosedList::ClosedList() : 
    states(initial_size),
    nrStates(0) 
{}

bool ClosedList::insert(State* state) {
    pair<hash_t::iterator, bool> pos = states.insert(make_pair(state->hashKey(), list_t()));
    
    list_t& l = pos.first->second;
    for (list_t::iterator it = l.begin(); it != l.end(); ++it) {
	if (!(*it)->discEqual(state)) {
	    continue;
	}
	state->share(*it);

	switch ((*it)->zone().relation(state->zone()))
	{
	case base_SUPERSET:  // state < *it
	case base_EQUAL:     // state == it
	    smallStates.push_front(state);
	    return false;
	case base_SUBSET:    // *it < state
	    smallStates.push_front(*it);
	    it = l.erase(it);
	    l.push_front(state);
	    ++nrStates;
	    return true;
	case base_DIFFERENT: // state & *it == 0
	    break;
	}
    }
    l.push_front(state);
    ++nrStates;
    return true;
}

bool ClosedList::contains(const State* state) const {
    hash_t::const_iterator pos = states.find(state->hashKey());
    if (pos == states.end()) {
	return false;
    }
    const list_t& l = pos->second;
    
    for (list_t::const_iterator it = l.begin(); it != l.end(); ++it) {
	if (!(*it)->discEqual(state)) {
	    continue;
	}
	switch ((*it)->zone().relation(state->zone()))
	{
	case base_SUPERSET:  // state < *it
	case base_EQUAL:     // state == it
	    return true;
	case base_SUBSET:    // *it < state
	case base_DIFFERENT: // state & *it == 0
	    break;
	}
    }
    return false;
}
