// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: normalize.h 44 2007-12-14 16:30:20Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef NORMALIZE_H
#define NORMALIZE_H

#include "system/state.h"
#include "system/system.h"

#include <vector>
#include <inttypes.h>

class System;
class Process;
class Location;
class ClockConstraint;

class Normalizer {
protected: 
    const System* system;   
    Normalizer(const System* system) : system(system) {}

    virtual void init();
    virtual void process(const Process*, const Location*, const ClockConstraint*) = 0;
public:
    virtual ~Normalizer() {}    
    virtual void normalize(State* next) const = 0;
};

////////////////////////////////////////////////////////////////////

class MaxBoundsNormalizer : public Normalizer {
protected: 
    std::vector<int32_t> bounds;

    virtual void process(const Process*, const Location*, const ClockConstraint* cc);
public:
    MaxBoundsNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class DiagonalMaxBoundsNormalizer : public MaxBoundsNormalizer {
public:
    DiagonalMaxBoundsNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class LUNormalizer : public Normalizer {
protected:
    std::vector<int32_t> lower;
    std::vector<int32_t> upper;

    virtual void process(const Process*, const Location*, const ClockConstraint* cc);    
public:
    LUNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class DiagonalLUBoundsNormalizer : public LUNormalizer {
public:
    DiagonalLUBoundsNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class LocalMaxBoundsNormalizer : public Normalizer {
protected: 
    typedef std::vector<std::vector<std::vector<int32_t> > > array_t;
    array_t bounds;
    uint32_t nrClocks;

    virtual void close(const Process* proc);    
    virtual void reserveMemory(array_t& array) const;
    virtual void process(const Process* proc, const Location* loc, const ClockConstraint* cc);    
    virtual void prepareNormalization(std::vector<int32_t>& ceils, const State* next) const;
public:
    LocalMaxBoundsNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class LocalDiagonalMaxBoundsNormalizer : public LocalMaxBoundsNormalizer {
public:
    LocalDiagonalMaxBoundsNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class LocalLUNormalizer : public Normalizer {
protected:
    typedef std::vector<std::vector<std::vector<int32_t> > > array_t;
    array_t lower;
    array_t upper;
    const uint32_t nrClocks;
    const uint32_t nrProcs;

    virtual void close(array_t& bounds, const Process* proc);    
    virtual void reserveMemory(array_t& array) const;
    virtual void process(const Process* proc, const Location* loc, const ClockConstraint* cc);    
    virtual void prepareNormalization(std::vector<int32_t>& ceils, const array_t& bounds, const State* next) const;

public:
    LocalLUNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

////////////////////////////////////////////////////////////////////

class LocalDiagonalLUNormalizer : public LocalLUNormalizer {
public:
    LocalDiagonalLUNormalizer(const System* system);
    virtual void normalize(State* next) const;
};

#endif /* NORMALIZE_H */
