// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: openlist.cpp 87 2008-01-16 17:20:24Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "search/openlist.h"
#include "system/state.h"
#include <climits>

using namespace std;

StackList::StackList() : 
    states() 
{}

void StackList::insert(State* state, int32_t) {
    states.push(state);
}

State* StackList::get() {
    State* next = states.top();
    states.pop();
    return next;
}

////////////////////////////////////////////////////////////////////

QueueList::QueueList() : 
    states() 
{}

void QueueList::insert(State* state, int32_t) {
    states.push(state);
}

State* QueueList::get() {
    State* next = states.front();
    states.pop();
    return next;
}

////////////////////////////////////////////////////////////////////

PriorityQueue::PriorityQueue() : 
    buckets(),
    lowest_bucket(0),
    nrStates(0)
{}

void PriorityQueue::insert(State* state, int32_t key) {    
    assert(key >= 0);
    if (key == INT_MAX) {
	// NOTE: do not delete the state, since a pointer to this
	// state can still be in use (e.g. DiscretePartCache)
	return;
    }
    if (key >= int32_t(buckets.size())) {
	buckets.resize(key + 1);
    }
    else if (key < lowest_bucket) {
	lowest_bucket = key;
    }
    buckets[key].push_back(state);
    nrStates++;
}

State* PriorityQueue::get() {
    assert(nrStates > 0);
    while (buckets[lowest_bucket].empty()) {
	lowest_bucket++;
    }
    nrStates--;
    // State* result = buckets[lowest_bucket].front();
    // buckets[lowest_bucket].pop_front();

    State* result = buckets[lowest_bucket].back();
    buckets[lowest_bucket].pop_back();

    return result;
}

