// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: openlist.h 283 2009-01-30 14:52:16Z Sebastian Kupferschmid $
//
////////////////////////////////////////////////////////////////////

#ifndef OPENLIST_H
#define OPENLIST_H

#include <stack>
#include <queue>
#include <vector>
#include <inttypes.h>

class State;

class OpenList {
protected:
    OpenList() {}
    virtual ~OpenList() {}
public:
    virtual void insert(State* state, int32_t) = 0;
    virtual State* get() = 0;
    virtual bool empty() const = 0;
    virtual uint32_t size() const = 0;
};

////////////////////////////////////////////////////////////////////

class StackList : public OpenList {
private:
    std::stack<State*> states;
public:
    StackList();
    virtual void insert(State* state, int32_t);
    virtual State* get();
    virtual bool empty() const { return states.empty(); }
    virtual uint32_t size() const { return states.size(); }
};

////////////////////////////////////////////////////////////////////

class QueueList : public OpenList {
private:
    std::queue<State*> states;
public:    
    QueueList();
    virtual void insert(State* state, int32_t);
    virtual State* get();
    virtual bool empty() const { return states.empty(); }
    virtual uint32_t size() const { return states.size(); }
};

////////////////////////////////////////////////////////////////////

class PriorityQueue : public OpenList {
private:
    typedef std::deque<State*> bucket_t;
    std::vector<bucket_t> buckets;
    mutable int32_t lowest_bucket;
    uint32_t nrStates;
public:
    PriorityQueue();

    virtual void insert(State* state, int32_t key);
    virtual State* get();
    
    virtual bool empty() const { return nrStates == 0; }
    virtual uint32_t size() const { return nrStates; }
};

#endif /* OPENLIST_H */
