// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: searchengine.h 95 2008-01-20 12:42:18Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef SEARCH_ENGINE_H
#define SEARCH_ENGINE_H

#include <inttypes.h>
#include <iosfwd>

class Task;
class State;
class OpenList;
class ClosedList;
class Heuristic;
class Normalizer;
class Options;
class Trace;

class SearchEngine {
public:
    enum status_t { UNKNOWN=0, SAT, UNSAT };
protected:
    const Options& opts;
    const Task* task;
    State* initial;
    const State* goal;
    OpenList* open;
    ClosedList* closed;
    Heuristic* heur;
    Normalizer* norm;

    status_t stat;
    uint32_t exploredStates;
    uint32_t generatedStates;
    
    std::ostream& printProgress(std::ostream&) const;
    std::ostream& printTrace(std::ostream&) const;
    std::ostream& printStatus(std::ostream&) const;
    void generateTrace(Trace& trace, const State* goal) const;    
public:    
    SearchEngine(const Options& opts, const Task* task);
    
    void init(OpenList*, ClosedList*, Heuristic*, Normalizer*);
    void explore();
    std::ostream& report(std::ostream&) const;
};

#endif /* SEARCHENGINE_H */
