// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: transition.cpp 73 2008-01-15 16:33:14Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "search/transition.h"

#include "system/system.h"
#include "system/process.h"
#include "system/edge.h"

#include <ostream>
#include <cassert>

using namespace std;

Transition::Transition(const Edge* edge1, const Edge* edge2) :
    edge1(edge1),
    edge2(edge2) 
{
    assert(edge1);
}

ostream& Transition::display(ostream& o) const {
    o <<  "    " << *edge1;
    if (edge2) {
	o << endl << "    " << *edge2;
    }    
    return o;
}

////////////////////////////////////////////////////////////////////

TransitionBuilder& TransitionBuilder::builder(const System* system) {
    static TransitionBuilder tb(system);
    return tb;
}

TransitionBuilder::TransitionBuilder(const System* system) : 
    nrEdges(system->getTotalNrEdges()),
    trans(nrEdges, vector<Transition>())
{
    // get all edges of the system
    vector<const Edge*> edges;
    for (uint32_t p = 0; p < system->getNrProcesses(); p++) {
	const Process* proc = system->getProcess(p);
	for (uint32_t e = 0; e < proc->getNrEdges(); e++) {
	    edges.push_back(proc->getEdge(e));	    
	}
    }
    assert(edges.size() == nrEdges);
    
    for (uint32_t i = 0; i < nrEdges; i++) {
	trans[i].reserve(nrEdges+1);
	for (uint32_t t = 0; t < nrEdges; t++) {
	    assert(edges[i]->idInSystem == i && edges[t]->idInSystem == t);
	    trans[i].push_back(Transition(edges[i], edges[t]));
	}
	trans[i].push_back(Transition(edges[i], NULL));
    }
}


