// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: transition.h 73 2008-01-15 16:33:14Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef TRANSITION_H
#define TRANSITION_H

#include <vector>
#include <iosfwd>
#include "system/edge.h"

class Edge;
class System;

class Transition {
private:
    Transition();
    Transition(const Edge* edge1, const Edge* edge2);
public:
    const Edge* edge1;
    const Edge* edge2;

    std::ostream& display(std::ostream& o) const;
    friend class TransitionBuilder;
};

inline std::ostream& operator<<(std::ostream& o, const Transition& trans) {
    return trans.display(o);
}

////////////////////////////////////////////////////////////////////

class TransitionBuilder {
    typedef std::vector<std::vector<Transition> > map_t;
    const uint32_t nrEdges;
    map_t trans;

    TransitionBuilder(const System* system);
    TransitionBuilder(const TransitionBuilder&);
    TransitionBuilder& operator=(const TransitionBuilder&);
public:
    
    static TransitionBuilder& builder(const System* system=0);
    const Transition* newTransition(const Edge* edge1, const Edge* edge2) const;
    const Transition* newTransition(const Edge* edge1) const;
};

////////////////////////////////////////////////////////////////////

inline const Transition* TransitionBuilder::newTransition(const Edge* edge1) const {
    return &trans[edge1->idInSystem][nrEdges];
}

inline const Transition* TransitionBuilder::newTransition(const Edge* edge1, const Edge* edge2) const {
    return &trans[edge1->idInSystem][edge2->idInSystem]; 	
}

#endif /* TRANSITION_H */
