// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: assignment.cpp 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "system/assignment.h"
#include "system/variable.h"
#include "system/state.h"
#include "common/helper.h"
#include "common/message.h"
#include <cassert>
#include <ostream>

using namespace std;

uint32_t IntAssignment::nrIntAssignments = 0;

vector<VarAssignment*> VarAssignment::all;

VarAssignment::VarAssignment(Integer* lhs, Integer* rhs, uint32_t id) : 
    IntAssignment(lhs),
    rhs(rhs),
    id(id)
{
    assert(lhs && rhs);
}

VarAssignment* VarAssignment::create(Integer* lhs, Integer* rhs) {
    for (uint32_t i = 0; i < all.size(); i++) {
	if (all[i]->lhs->id == lhs->id && all[i]->rhs->id == rhs->id) {
	    return all[i];
	}	
    }
    all.push_back(new VarAssignment(lhs, rhs, nrIntAssignments++));
    return all.back();
}

void VarAssignment::apply(State* state) const {
    /// @todo check if the assigned value is in lhs's domain
    state->setIntegerValue(lhs->id, state->getIntegerValue(rhs->id));
}

ostream& VarAssignment::display(ostream& o) const {
    return o << "(assign " << lhs->name << " := " << rhs->name << ")"; 
}

////////////////////////////////////////////////////////////////////

vector<ValueAssignment*> ValueAssignment::all;

ValueAssignment::ValueAssignment(Integer* lhs, int32_t rhs, uint32_t id) :
    IntAssignment(lhs),
    rhs(rhs),
    id(id)
{
    init();
}

ValueAssignment::ValueAssignment(Integer* lhs, const Constant* rhs, uint32_t id) :
    IntAssignment(lhs),
    rhs(rhs->value),
    id(id)
{
    init();
}

void ValueAssignment::init() const {
    assert(lhs);
    if (rhs > lhs->upper || rhs < lhs->lower) {
	error() << rhs << " is outside " << lhs->name << "'s domain" << endl;
    }
}

ValueAssignment* ValueAssignment::create(Integer* lhs, int32_t rhs) {
    for (uint32_t i = 0; i < all.size(); i++) {
	if (all[i]->lhs->id == lhs->id && all[i]->rhs == rhs) {
	    return all[i];
	}
    }
    all.push_back(new ValueAssignment(lhs, rhs, nrIntAssignments++));
    return all.back();
}

ValueAssignment* ValueAssignment::create(Integer* lhs, const Constant* rhs) {
    return create(lhs, rhs->value);
}


void ValueAssignment::apply(State* state) const {
    state->setIntegerValue(lhs->id, rhs);
}

ostream& ValueAssignment::display(ostream& o) const {
    return o << "(assign " << lhs->name << " := " << rhs << ")"; 
}

////////////////////////////////////////////////////////////////////

vector<ClockReset*> ClockReset::all;

ClockReset::ClockReset(Clock* lhs, int32_t rhs, uint32_t id) :
    lhs(lhs),
    rhs(rhs),
    id(id)
{
    assert(lhs && rhs == 0);
}

ClockReset::ClockReset(Clock* lhs, const Constant* rhs, uint32_t id) :
    lhs(lhs),
    rhs(rhs->value),
    id(id)
{
    assert(lhs && rhs == 0);
}

ClockReset* ClockReset::create(Clock* lhs, int32_t rhs) {
    assert(rhs == 0);
    for (uint32_t i = 0; i < all.size(); i++) {
	if (all[i]->lhs->id == lhs->id) {
	    return all[i];
	}
    }
    all.push_back(new ClockReset(lhs, rhs, all.size()));
    return all.back();
}

ClockReset* ClockReset::create(Clock* lhs, const Constant* rhs) {
    return create(lhs, rhs->value);
}

void ClockReset::apply(State* state) const {
    state->zone()(lhs->id) = 0;
}

ostream& ClockReset::display(ostream& o) const {
    return o << "(reset " << lhs->name << " := " << rhs << ")"; 
}


