// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: assignment.h 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef ASSIGNMENT_H
#define ASSIGNMENT_H

#include "system/expression.h"
#include <vector>

class Integer;
class Clock;
class Constant;
class State;

class Assignment : public Expression {    
protected:
    Assignment() : Expression(Expression::ASSIGNMENT) {}
public:
    virtual void apply(State* state) const = 0;
};

////////////////////////////////////////////////////////////////////

class IntAssignment : public Assignment {    
public:
    static uint32_t nrIntAssignments;
    const Integer* lhs;
protected:
    IntAssignment(Integer* lhs) : Assignment(), lhs(lhs) {}    
};

////////////////////////////////////////////////////////////////////

class VarAssignment : public IntAssignment {
private:
    static std::vector<VarAssignment*> all;

    VarAssignment(Integer* lhs, Integer* rhs, uint32_t id);
public:
    const Integer* rhs;
    const uint32_t id;
    
    static VarAssignment* create(Integer* lhs, Integer* rhs);

    virtual void apply(State* state) const;
    virtual std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

class ValueAssignment : public IntAssignment {
private:
    static std::vector<ValueAssignment*> all;

    ValueAssignment(Integer* lhs, int32_t rhs, uint32_t id);
    ValueAssignment(Integer* lhs, const Constant* rhs, uint32_t id);

    void init() const;
public:
    const int32_t rhs;
    const uint32_t id;

    static ValueAssignment* create(Integer* lhs, int32_t rhs);
    static ValueAssignment* create(Integer* lhs, const Constant* rhs);
    
    virtual void apply(State* state) const;
    virtual std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

class ClockReset : public Assignment {
private:
    static std::vector<ClockReset*> all;

    ClockReset(Clock* lhs, int32_t rhs, uint32_t id);
    ClockReset(Clock* lhs, const Constant* rhs, uint32_t id);    
public:
    const Clock* lhs;
    const int32_t rhs;
    const uint32_t id;
    
    static ClockReset* create(Clock* lhs, int32_t rhs);
    static ClockReset* create(Clock* lhs, const Constant* rhs);    

    virtual void apply(State* state) const;
    virtual std::ostream& display(std::ostream& o) const;
};

#endif /* ASSIGNMENT_H */
