// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: component.h 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef COMPONENT_H
#define COMPONENT_H

#include "common/utapfwd.h"
#include <vector>
#include <iosfwd>
#include <string>

class Variable;
class Integer;
class Clock;
class Channel;
class Constant;

class Component {
protected:
    Component(const std::string& name, Component* parent);
    virtual ~Component() {}
public:
    std::vector<Variable*> children;
    std::vector<Constant*> consts;
    std::vector<Integer*> ints;
    std::vector<Clock*> clocks;
    std::vector<Channel*> channels;     

    const Component* parent;
    const std::string name;
    
    virtual void addInteger(Integer* integer);
    virtual void addClock(Clock* clock);
    virtual void addChannel(Channel* channel);
    virtual void addConstant(Constant* constant);   

    virtual Integer* getInteger(uint32_t index) const;
    virtual uint32_t getNrIntegers() const;
    
    virtual Clock* getClock(uint32_t index) const;
    virtual uint32_t getNrClocks() const;
    
    virtual Channel* getChannel(uint32_t index) const;
    virtual uint32_t getNrChannels() const;

    virtual const Component* getComponent(const std::string& compname) const;
    virtual Variable* getChild(uint32_t index) const;
    virtual std::ostream& display(std::ostream& o) const;

    virtual Constant* findConst(const UTAP::expression_t& expr) const;
    virtual Variable* find(const std::string& name) const;
    virtual Variable* get(uint32_t index) const;
};

inline std::ostream& operator<<(std::ostream& o, const Component& c) {
    return c.display(o);
}

#endif /* COMPONENT_H */
