// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: edge.cpp 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "system/edge.h"
#include "system/variable.h"
#include "system/location.h"
#include "system/effect.h"
#include "system/guard.h"
#include "system/process.h"
#include "system/state.h"

#include <ostream>
#include <cassert>

using namespace std;

uint32_t Edge::nrEdges = 0;

Edge::Edge(int32_t idInProcess) : 
    action(NULL),
    type(TAU),
    idInProcess(idInProcess),
    idInSystem(nrEdges++), 
    src(NULL),
    dst(NULL),
    effect(NULL),
    guard(NULL)
{}

Edge::~Edge() {
    delete action;
    delete guard;
    delete effect;   
}

void Edge::setAction(Action* action) {    
    if (!action) {
	type = TAU;
    }
    else if (action->isBang) {
	type = BANG; 
    } else {
	type = QUE;
    }
    this->action = action;
}

const Process* Edge::getProcess() const {
    assert(src->proc->id == dst->proc->id);
    return src->proc;
}

ostream& Edge::display(ostream& o) const {
    assert(guard);
    o << src->proc->name << "." << src->name << " -> " 
      << dst->proc->name << "." << dst->name << " {";     
    o << *guard << " ";
    if (action) {
	o << *action;
    } 
    return o << " " << *effect << "}"; 
}
