// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: edge.h 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef EDGE_H
#define EDGE_H

#include <iosfwd>
#include <inttypes.h>

class Location;
class Guard;
class Effect;
class Action;
class Process;
class State;

class Edge {
public:
    enum type_t { TAU = 0, BANG, QUE };
private:
    static uint32_t nrEdges;
    Action* action;
    type_t type;
public:

    const uint32_t idInProcess;
    const uint32_t idInSystem;
    Location* src;
    Location* dst;
    Effect* effect;
    Guard* guard;
    
    Edge(int32_t idInProcess);
    ~Edge();
    
    const Action* getAction() const;
    void setAction(Action* action);
    type_t getType() const;
    const Process* getProcess() const;
    
    std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

inline const Action* Edge::getAction() const {    
    return action;
}

inline Edge::type_t Edge::getType() const {
    return type;
}

inline std::ostream& operator<<(std::ostream& o, const Edge& edge) {
    return edge.display(o);
}

#endif /* EDGE_H */
