// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: expressionbuilder.h 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef EXPRESSION_BUILDER_H
#define EXPRESSION_BUILDER_H

#include "common/utapfwd.h"
#include <inttypes.h>

class Component;
class Expression;
class Effect;
class Variable;
class Guard;
class Invariant;
class Action;
class Target;
class System;
class Constraint;
class Assignment;

class ExpressionBuilder {
private:
    Expression* build(const UTAP::expression_t& expr, const Component* comp, Expression* result) const;
    Variable* resolve(const UTAP::expression_t& expr, const Component* proc) const;
    Constraint* buildConstraint(const UTAP::expression_t& exp, const Component* comp, Expression* result) const;
    Assignment* buildAssignment(const UTAP::expression_t& exp, const Component* comp, Expression* result) const;
    Variable* buildDotExpression(const UTAP::expression_t& exp, const Component* comp, Expression* result) const;
public:
    ExpressionBuilder() {}

    int32_t evaluate(const UTAP::expression_t& exp, const Component* comp) const;    

    Guard* buildGuard(const UTAP::expression_t& exp, const Component* comp) const;
    Target* buildTarget(const char* filename, UTAP::TimedAutomataSystem* tasystem, const System* system) const;
    Effect* buildEffect(const UTAP::expression_t& exp, const Component* comp) const;
    Action* buildAction(const UTAP::expression_t& exp, const Component* comp) const;
    Invariant* buildInvariant(const UTAP::expression_t& exp, const Component* comp) const;
};

#endif /* EXPRESSION_BUILDER_H */
