// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: location.cpp 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "system/location.h"
#include "system/process.h"
#include "system/invariant.h"
#include "system/edge.h"
#include <ostream>
#include <cassert>

using namespace std;

uint32_t Location::nrLocations = 0;

Location::Location(Process* proc, const std::string& name, int32_t idInProcess) :
    Variable(name, Expression::LOCATION),
    proc(proc),
    urgent(false),
    commit(false),
    idInSystem(nrLocations++),
    idInProcess(idInProcess),
    inv(NULL),
    outgoing()
{}

Location::~Location() {
    delete inv;
}

uint32_t Location::getNrEdges() const {
    return outgoing.size();
}

Edge* Location::getEdge(uint32_t index) const {
    assert(index < outgoing.size());
    return outgoing[index];
}

ostream& Location::display(ostream& o) const {
    assert(inv);
    return o << idInProcess << ": " << proc->name << "." << name << *inv;
}
