// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: parser.cpp 283 2009-01-30 14:52:16Z Sebastian Kupferschmid $
//
////////////////////////////////////////////////////////////////////

#include "system/parser.h"
#include "system/task.h"
#include "system/system.h"
#include "system/target.h"
#include "system/systembuilder.h"
#include "system/expressionbuilder.h"
#include "common/message.h"
#include <utap/utap.h>

#include <fstream>
#include <vector>
#include <cstring>

using namespace std;

static bool check_file(const char* filename) {
    FILE* file = fopen(filename, "r");
    if (!file) {
	error() << "file '" << filename << "' does not exist" << endl;
	return false;
    }
    fclose(file);
    return true;
}

Task* SystemParser::parse(const string& tafile, const string& qfile, bool oldSyntax) {
    return parse(tafile.c_str(), qfile.c_str(), oldSyntax);
}

Task* SystemParser::parse(const char* tafile, const char* qfile, bool oldSyntax) {
    check_file(tafile);
    check_file(qfile);

    UTAP::TimedAutomataSystem tasystem;
    if (strlen(tafile) > 4 && strcasecmp(".xml", tafile + strlen(tafile) - 4) == 0) {
        parseXMLFile(tafile, &tasystem, !oldSyntax);
    } else {
        FILE *file = fopen(tafile, "r");       
        parseXTA(file, &tasystem, !oldSyntax);           
        fclose(file);
    }
    vector<UTAP::error_t>::const_iterator it;
    const vector<UTAP::error_t> &errors = tasystem.getErrors();
    const vector<UTAP::error_t> &warns = tasystem.getWarnings();
    
    for (it = errors.begin(); it != errors.end(); it++) {
        cerr << "error: " << *it << endl;
    }
    for (it = warns.begin(); it != warns.end(); it++) {
        cerr << "waring: " << *it << endl;
    }
    if (!errors.empty()) {            
        return NULL;
    }

    SystemBuilder builder;
    ExpressionBuilder ebuilder;
    System* system = builder.buildSystem(&tasystem);
    Target* target = ebuilder.buildTarget(qfile, &tasystem, system);

    return new Task(system, target);
}
