// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: process.cpp 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "system/process.h"
#include "system/location.h"
#include "system/edge.h"
#include "common/helper.h"
#include <ostream>
#include <cassert>

using namespace std;

uint32_t Process::nrProcesses = 0;

Process::Process(const string& name, Component* parent) : 
    Component(name, parent),
    locs(),
    edges(),
    init(NULL),
    id(nrProcesses++)
{}

Process::~Process() {
    deleteVector(edges);
    deleteVector(locs);
}

Location* Process::getLocation(uint32_t index) const {
    assert(index < locs.size());
    return locs[index];
}

uint32_t Process::getNrLocations() const {
    return locs.size();
}

Edge* Process::getEdge(uint32_t index) const {
    assert(index < edges.size());
    return edges[index];
}

uint32_t Process::getNrEdges() const {
    return edges.size();
}

void Process::addEdge(Edge* edge) {
    edges.push_back(edge);
}

Location* Process::getLocationByName(const string& locname) const {
    for (uint32_t i = 0; i < locs.size(); i++) {
	assert(locs[i]);
	if (locs[i]->name == locname) {
	    return locs[i];
	}
    }
    return NULL;
}

void Process::addLocation(Location* loc) {
    assert(loc);
    loc->proc = this;
    locs.push_back(loc);
    children.push_back(loc);
}

ostream& Process::display(ostream& o) const {
    o << "process " << name << " {" << endl;
    Component::display(o);
    o << endl;
    o << "    states ";
    printPtrVector(locs, ", ", o);
    o << ";" << endl;    
    o << "    init " << *init << endl; 
    o << "    edges" << endl << "    ";
    printPtrVector(edges, "\n    ", o);
    return o << "}";
}
