// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: state.cpp 102 2008-01-22 13:27:25Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "system/state.h"
#include "system/system.h"
#include "system/process.h"
#include "system/location.h"
#include "system/variable.h"

#include "common/helper.h"

#include <dbm/ClockAccessor.h>

#include <iostream>
#include <cassert>

using namespace std;

uint32_t State::nrClocks = 0;
uint32_t DiscretePart::nrProcs = 0;
uint32_t DiscretePart::nrInts = 0;
uint32_t DiscretePart::size = 0;
StateBuilder* State::builder = NULL;


StateBuilder::StateBuilder(const System* system) : 
    system(system),
    nrClocks(system->getTotalNrClocks()),
    nrProcs(system->getNrProcesses()),
    nrInts(system->getTotalNrIntegers()),
    discreteSize(nrProcs + nrInts),
    cap(0)
{
    DiscretePart::nrInts = nrInts;
    DiscretePart::nrProcs = nrProcs;
    DiscretePart::size = discreteSize;   
    State::nrClocks = nrClocks;
}

ostream& State::prettyPrint(ostream& o, const System* system) const {
    dp.prettyPrint(o, system);
    return o << endl << zone().toString(MyClockAccessor(system));
}

ostream& State::display(ostream& o) const {
    return o << "(" << dp << ")"; 
}

std::ostream& DiscretePart::display(std::ostream& o) const {
    return printContainer(data, data + size, " ", o);    
}

std::ostream& DiscretePart::prettyPrint(std::ostream& o, const System* system) const {
    for (uint32_t i = 0; i < system->getNrProcesses(); i++) {
	const Process* proc = system->getProcess(i);
 	o << proc->name << "." << proc->getLocation(getLocationNr(i))->name << " ";
    }
    o << endl;
    uint32_t pos = 0;
    for (uint32_t i = 0; i < system->getNrIntegers(); i++) {
	o << system->getInteger(i)->name << "=" << getIntegerValue(pos++) << " ";
    }
    
    for (uint32_t j = 0; j < system->getNrProcesses(); j++) {
	const Process* proc = system->getProcess(j);
	for (uint32_t l = 0; l < proc->getNrIntegers(); l++) {
	    o << proc->name << "." << proc->getInteger(l)->name << "=" << getIntegerValue(pos++) << " ";
	}
    }
    return o;
}

