// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: target.cpp 43 2007-12-14 14:25:39Z sebastian $
//
////////////////////////////////////////////////////////////////////

#include "system/target.h"
#include "system/guard.h"
#include "system/location.h"
#include "system/state.h"
#include "system/process.h"
#include "common/message.h"
#include "common/helper.h"
#include <ostream>
#include <cassert>

using namespace std;

Target::Target() : 
    ConstraintContainer(Expression::TARGET),
    locConstraints(),
    intConstraints()
{}

Target::~Target() {
    // no need to delete the locs, they will be deleted by the
    // processes
    intConstraints.destroy();
}

void Target::addLocationConstraint(LocationConstraint* lc) {
    locConstraints.add(lc);
}

void Target::addIntConstraint(IntConstraint* ic) {
    intConstraints.add(ic);
}

void Target::addClockConstraint(ClockConstraint*) {
    error() << "clock constraints are NOT allowed in targets" << endl;
}

ostream& Target::display(std::ostream& o) const {
    if (locConstraints.empty()) {
	return o << intConstraints;
    } else {
	return o << locConstraints << " " << intConstraints;
    }
}

