// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: target.h 48 2008-01-07 17:09:18Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef TARGET_H
#define TARGET_H

#include "system/constraintcontainer.h"
#include "system/constraint.h"

class Target : public ConstraintContainer {
private:
    Conjunction<LocationConstraint> locConstraints;
    Conjunction<IntConstraint> intConstraints;

public:    
    Target();
    virtual ~Target();

    virtual void addIntConstraint(IntConstraint* ic);
    virtual void addClockConstraint(ClockConstraint* cc);
    virtual void addLocationConstraint(LocationConstraint* lc);
    virtual bool isSatBy(const State* state) const;

    const Conjunction<LocationConstraint>& getLocationConstraints() const {
	return locConstraints;
    }
    
    const Conjunction<IntConstraint>& getIntConstraints() const {
	return intConstraints;
    }

    virtual std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

inline bool Target::isSatBy(const State* state) const {
    return locConstraints.isSatBy(state) && intConstraints.isSatBy(state);
}

#endif /* TARGET_H */
