// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; -*-
////////////////////////////////////////////////////////////////////
//
// $Id: variable.h 59 2008-01-11 15:26:44Z sebastian $
//
////////////////////////////////////////////////////////////////////

#ifndef VARIABLE_H
#define VARIABLE_H

#include "system/expression.h"
#include <inttypes.h>
#include <map>
#include <string>

/** Variable is the base class for all variabes, e.g. Integers, Clocks.
 **/
class Variable : public Expression {
public:
    const std::string name;

    Variable(const std::string& name, Expression::type_t type) : 
	Expression(type), 
	name(name) 
	{}
};

////////////////////////////////////////////////////////////////////

class Channel : public Variable {
private:
    static uint32_t nrChannels;
public:
    bool urgent;
    bool broadcast;
    const uint32_t id;
	
    explicit Channel(const std::string& name);

    virtual std::ostream& display(std::ostream& o) const;    
};

////////////////////////////////////////////////////////////////////

/* An Action is mainly a pointer to a Channel. It has the same id as
 * the Channel.
 */
class Action : public Variable {
public:
    Channel* chan;
    const bool isBang;
    const uint32_t id;
    
    Action(Channel* chan, bool isBang);
    virtual std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

class Clock : public Variable {
private:
    static uint32_t nrClocks;
public:
    const uint32_t id;
    explicit Clock(const std::string& name);

    virtual std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

class Constant : public Variable {
public:
    const int32_t value;
    
    explicit Constant(const std::string& name);
    Constant(const std::string& name, int32_t value);

    virtual std::ostream& display(std::ostream& o) const;
};

////////////////////////////////////////////////////////////////////

/** As there are normaly many Constants representing the same value,
 * prefere to use this factory for creating Constants.
 **/
class ConstantFactory {
private:
    typedef std::map<int32_t, Constant*> map_t;
    map_t constants;
    
    ConstantFactory();
    ~ConstantFactory();    
public:
    static Constant* newConstant(int32_t value);
};

////////////////////////////////////////////////////////////////////

class Integer : public Variable {
private:
    static uint32_t nrIntegers;
public:
    static const int32_t MIN_BOUND = -32767;
    static const int32_t MAX_BOUND = 32767;
    
    int32_t lower;
    int32_t upper;
    int32_t initial;
    const uint32_t id;
    
    explicit Integer(const std::string& name);    
    virtual std::ostream& display(std::ostream& o) const;
};


#endif /* VARIABLE_H */
